import { CoreRunner } from './core/CoreRunner';
export declare type IDoneFunction<T = any> = (result?: T) => void;
export declare type ITaskFunction<T = any> = (done: IDoneFunction<T>) => void;
export interface ITasks<T = any> {
    total: number;
    completed: number;
    running: number;
    list: ITaskFunction<T>[];
}
export declare const RemovalMethods: {
    readonly ALL: "all";
    readonly SINGULAR_LAST: "singular-last";
    readonly SINGULAR_FIRST: "singular-first";
    readonly AT_INDEX: "at-index";
    readonly RANGE: "range";
};
export declare type IRemovalMethods = typeof RemovalMethods[keyof typeof RemovalMethods];
export interface IDuration {
    start?: Date;
    end?: Date;
    total?: number;
}
export declare type IOnStart<T = any> = ({ tasks, duration, }: {
    tasks: ITasks<T>;
    duration: IDuration;
}) => void;
export declare type IOnAdd<T = any> = ({ tasks }: {
    tasks: ITasks<T>;
}) => void;
export declare type IOnRemove<T = any> = ({ tasks, method, removedTasks, }: {
    tasks: ITasks<T>;
    method: IRemovalMethods;
    removedTasks: ITaskFunction<T>[];
}) => void;
export declare type IOnRun<T = any> = ({ task, tasks, duration, }: {
    task: ITaskFunction<T>;
    tasks: ITasks<T>;
    duration: IDuration;
}) => void;
export declare type IOnDone<T = any> = ({ tasks, result, }: {
    tasks: ITasks<T>;
    result?: T;
}) => void;
export declare type IOnEnd<T = any> = ({ tasks, duration, }: {
    tasks: ITasks<T>;
    duration: IDuration;
}) => void;
export interface IRunnerOptions<T = any> {
    concurrency: number;
    autoStart: boolean;
    name: string | (() => string);
    onAdd: IOnAdd<T>;
    onRemove: IOnRemove<T>;
    onStart: IOnStart<T>;
    onRun: IOnRun<T>;
    onDone: IOnDone<T>;
    onEnd: IOnEnd<T>;
}
export declare type IStrategy<T = any> = (this: CoreRunner, task: ITaskFunction<T>, done: IDoneFunction<T>) => void;
